package org.example.spaceinvaders;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Typeface;
import android.graphics.drawable.Drawable;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.media.AudioManager;
import android.media.SoundPool;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;

import java.util.List;
import java.util.Vector;

/**
 * Created by ana on 14/11/15.
 */

public class VistaJuego extends View implements SensorEventListener {

    // MARCIANOS
    private Marcianos marcianos; // Vector con los Marcianos
    private static final int NMARCIANOS = 55; // Número inicial de marcianos
    private static final int NMARCIANOS_COL = 11; //Número de marcianos en cada columna
    private static final int NCANGREJOS_FILA = 2;
    private static final int NCALAMARES_FILA = 2;
    private static final int NPULPOS_FILA = 1;


    private int PERIODO_PROCESO_MARCIANO = 300;
    private long ultimoProcesoMarciano = 0;

    // NAVE
    private Grafico nave; // Gráfico de la nave
    private static final int VELOCIDAD_NAVE = 1;

    // MISIL
    private Vector<Grafico> misiles;
    private static int VELOCIDAD_MISIL = 12;
    private Drawable drawableMisil;

    //MISILES Marcianos
    private Vector<Grafico> misilesMarcianos;
    private int NMAXMISILESMARCIANOS = 2;


    // //// THREAD Y TIEMPO //////
    // Thread encargado de procesar el juego
    private ThreadJuego thread = new ThreadJuego();

    // Cada cuanto queremos procesar cambios (ms)
    private static int PERIODO_PROCESO = 50;

    // Cuando se realizó el último proceso
    private long ultimoProceso = 0;

    //Sesnsores
    SensorManager mSensorManager;

    //Preferencias
    SharedPreferences pref;

    //SoundPool en Marcianos
    SoundPool soundPool;
    int idDisparo, idExplosion, idGameOver;

    //Puntuaciones
    private Paint pincel = new Paint(); //Pincel para puntaciones
    private int puntuacion = 0;

    private boolean continuar=true;


    public VistaJuego(Context context, AttributeSet attrs) {

        super(context, attrs);

        //Inicializar preferencias
        pref = PreferenceManager.getDefaultSharedPreferences(getContext());


        //Preparar gráficos de marcianos
        marcianos = new Marcianos();

        //Crear las filas de marcianos

        for (int i = 0; i < NPULPOS_FILA * NMARCIANOS_COL; i++) {
            Pulpo pulpo = new Pulpo(this,
                    context.getResources().getDrawable(R.mipmap.pulpo1),
                    context.getResources().getDrawable(R.mipmap.pulpo2));
            marcianos.add(pulpo);
        }

        for (int i = 0; i < NCANGREJOS_FILA * NMARCIANOS_COL; i++) {
            Cangrejo cangrejo = new Cangrejo(this,
                    context.getResources().getDrawable(R.mipmap.cangrejo),
                    context.getResources().getDrawable(R.mipmap.cangrejo));
            marcianos.add(cangrejo);
        }

        for (int i = 0; i < NCALAMARES_FILA * NMARCIANOS_COL; i++) {
            Calamar calamar = new Calamar(this,
                    context.getResources().getDrawable(R.mipmap.calamar1),
                    context.getResources().getDrawable(R.mipmap.calamar2));
            marcianos.add(calamar);
        }
        //Preparar gráficos nave y misil
        Drawable drawableNave = context.getResources().getDrawable(
                R.mipmap.ship);
        nave = new Grafico(this, drawableNave);

        misiles = new Vector<Grafico>();
        misilesMarcianos = new Vector<Grafico>();

        pincel.setColor(Color.WHITE);
        pincel.setTextSize(20);
        pincel.setTypeface(Typeface.DEFAULT_BOLD);


        //Inicializar Sensores, si está seleccionado en preferencias
        if (pref.getString("entradas", "1").equals("1")) {

            mSensorManager = (SensorManager)
                    context.getSystemService(Context.SENSOR_SERVICE);

        }


        //Inicializar el SoundPool
        soundPool = new SoundPool(5, AudioManager.STREAM_MUSIC, 0);
        idDisparo = soundPool.load(context, R.raw.disparo, 0);
        idExplosion = soundPool.load(context, R.raw.explosion, 0);
        idGameOver = soundPool.load(context, R.raw.gameover, 0);


    }

    @Override
    protected void onSizeChanged(int ancho, int alto,
                                 int ancho_anter, int alto_anter) {
        super.onSizeChanged(ancho, alto, ancho_anter, alto_anter);

        // Una vez que conocemos nuestro ancho y alto.
        //Calculamos donde colocar la nave.
        nave.setCenX(ancho / 2);
        nave.setCenY(alto - nave.getAlto());
        nave.setIncX(VELOCIDAD_NAVE);

        marcianos.iniciarPosicionMarcianos(ancho, alto, NMARCIANOS_COL);

        ultimoProceso = System.currentTimeMillis();
        ultimoProcesoMarciano = System.currentTimeMillis();
        thread.start();
    }

    protected void actualizaFisica() {

        long ahora = System.currentTimeMillis();


        if (ultimoProceso + PERIODO_PROCESO > ahora) {
            return;
            // Salir si el período de proceso no se ha cumplido.
        }

        // Para una ejecución en tiempo real calculamos retardo
        double retardo = (ahora - ultimoProceso) / PERIODO_PROCESO;
        ultimoProceso = ahora; // Para la próxima vez

        double retardoMarciano = 0;


        if ((ahora - ultimoProcesoMarciano) > PERIODO_PROCESO_MARCIANO) {

            retardoMarciano = (ahora - ultimoProcesoMarciano) / PERIODO_PROCESO;
            ultimoProcesoMarciano = ahora;


            continuar = marcianos.incrementarPosicion(getWidth(), getHeight(), retardoMarciano);

            //Si los marcianos han llegado al final de la pantalla, el juego ha finalizado
            if (!continuar) {
                salir();
            }

        }
        if (continuar) {
            if ((ahora - ultimoProcesoMarciano) > (PERIODO_PROCESO_MARCIANO / NMAXMISILESMARCIANOS)) {

                //Elegimos aleatoriamente un marciano para que dispare.
                //Se admiten NMAXMISILESMARCIANOS marcianos disparando a la vez
                //Los marcianos disparan sus misiles
                if (misilesMarcianos.size() < NMAXMISILESMARCIANOS) {
                    int nmarcianodispara = (int) (Math.random() * marcianos.size());
                    activaMisilMarciano(marcianos.get(nmarcianodispara));
                }
            }

            // Actualizamos la posición de los misiles
            Grafico misil;
            for (int i = 0; i < misiles.size(); i++) {
                misil = misiles.get(i);
                misil.setCenY((int) (misil.getCenY() - misil.getIncY() * retardo));
                if ((misil.getCenY() - misil.getAlto() / 2) < 0) {
                    misiles.remove(i);

                } else {
                    for (int j = 0; j < marcianos.size(); j++) {
                        if (misiles.elementAt(i).verificaColision(marcianos.get(j))) {
                            destruyeMarciano(j, i);
                            break;
                        }
                    }
                }

            }

            // Actualizamos la posición de los misiles Marcianos
            Grafico misilMarciano;
            for (int i = 0; i < misilesMarcianos.size(); i++) {
                misilMarciano = misilesMarcianos.get(i);
                misilMarciano.setCenY((int) (misilMarciano.getCenY() + misilMarciano.getIncY() * retardo));
                if ((misilMarciano.getCenY() + misilMarciano.getAlto() / 2) > getHeight()) {
                    misilesMarcianos.remove(i);

                } else {
                    for (int j = 0; j < misilesMarcianos.size(); j++) {
                        if (misilesMarcianos.elementAt(j).verificaColision(nave)) {
                            continuar=false;
                            salir();
                        }
                    }
                }
            }
        }
    }

    public void incrementaPosNave(double factor) {
        if (((nave.getCenX() + nave.getIncX() * factor) > (nave.getAncho() / 2)) &&
                (nave.getCenX() + nave.getIncX() * factor) < getWidth() - (nave.getAncho() / 2)) {
            nave.setCenX((int) (nave.getCenX() + nave.getIncX() * factor));
        }
    }

    private float mX = 0, mY = 0;
    private boolean disparo = false;

    @Override
    public boolean onTouchEvent(MotionEvent event) {

        boolean procesada = false;
        super.onTouchEvent(event);
        float x = event.getX();
        float y = event.getY();
        switch (event.getAction()) {
            case MotionEvent.ACTION_DOWN:
                disparo = true;
                break;
            case MotionEvent.ACTION_MOVE:
                float dx = Math.abs(x - mX);
                if (dx > 5) {
                    incrementaPosNave(x - mX);
                    disparo = false;

                }
                break;
            case MotionEvent.ACTION_UP:
                if (disparo) {
                    activaMisil();
                }
                break;
        }
        mX = x;
        mY = y;
        procesada = true;
        return procesada;
    }

    @Override
    public void onAccuracyChanged(Sensor sensor, int accuracy) {
    }

    private boolean hayValorInicial = false;
    private float valorInicialX;
    private float valorInicialY;
    private float valorInicialZ;

    @Override
    public void onSensorChanged(SensorEvent event) {


        if (pref.getString("entradas", "1").equals("1")) {
            //Mover la nave
            float valorx = event.values[0];
            float valory = event.values[1];
            float valorz = event.values[2];
            if (!hayValorInicial) {
                valorInicialX = valorx;
                valorInicialY = valory;
                valorInicialZ = valorz;
                hayValorInicial = true;
            }
            incrementaPosNave((int) (valorInicialX - valorx));
        }
    }

    void activarSensores() {
        if (pref.getString("entradas", "1").equals("1")) {
            List<Sensor> listSensors = mSensorManager.getSensorList(
                    Sensor.TYPE_ACCELEROMETER);
            if (!listSensors.isEmpty()) {
                Sensor orientationSensor = listSensors.get(0);
                mSensorManager.registerListener(this, orientationSensor,
                        SensorManager.SENSOR_DELAY_GAME);
            }
        }

    }

    void desactivarSensores() {
        if (pref.getString("entradas", "1").equals("1")) {
            mSensorManager.unregisterListener(this);
        }
    }

    private void destruyeMarciano(int i_marciano, int i_misil) {
        synchronized (marcianos) {

            //Sonido de destrucción de marciano
            soundPool.play(idExplosion, 1, 1, 0, 0, 2);
            puntuacion += marcianos.get(i_marciano).getPuntuacion();
            marcianos.remove(i_marciano);
            misiles.remove(i_misil);
            if (marcianos.isEmpty()) {
                continuar=false;
                salir();
            }
        }
    }

    private void activaMisil() {

        drawableMisil = getContext().getResources().getDrawable(R.mipmap.misil1);
        Grafico misil = new Grafico(this, drawableMisil);
        misil.setCenX(nave.getCenX());
        misil.setCenY(nave.getCenY());
        misil.setIncY(VELOCIDAD_MISIL);
        misiles.add(misil);
        soundPool.play(idDisparo, 1, 1, 1, 0, 2);
    }

    private void activaMisilMarciano(Marciano marciano) {

        drawableMisil = getContext().getResources().getDrawable(R.mipmap.misilmarciano);
        Grafico misil = new Grafico(this, drawableMisil);
        misil.setCenX(marciano.getCenX());
        misil.setCenY(marciano.getCenY());
        misil.setIncY(VELOCIDAD_MISIL);
        misilesMarcianos.add(misil);
        soundPool.play(idDisparo, 1, 1, 1, 0, 2);
    }


    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        synchronized (marcianos) {
            Marciano marciano;
            for (int i = 0; i < marcianos.size(); i++) {
                marcianos.get(i).dibujaGrafico(canvas);
            }
            for (Grafico misil : misiles) {
                misil.dibujaGrafico(canvas);
            }
            for (Grafico misil : misilesMarcianos) {
                misil.dibujaGrafico(canvas);
            }
            canvas.drawText("SCORE:   " + puntuacion, 20, 30, pincel);
        }
        nave.dibujaGrafico(canvas);

    }

    void salir() {
        soundPool.play(idGameOver, 1, 1, 0, 0, 2);
        Bundle bundle = new Bundle();
        bundle.putInt("puntuacion", puntuacion);
        Intent intent = new Intent();
        intent.putExtras(bundle);
        thread.detener();
        padre.setResult(Activity.RESULT_OK, intent);
        padre.finish();
    }

    private Activity padre;

    public void setPadre(Activity padre) {
        this.padre = padre;
    }

    class ThreadJuego extends Thread {

        private boolean pausa, corriendo;

        public synchronized void pausar() {
            pausa = true;
        }

        public synchronized void reanudar() {
            pausa = false;
            notify();
        }

        public void detener() {
            corriendo = false;
            if (pausa) reanudar();
        }

        public ThreadJuego getThread() {
            return thread;
        }

        @Override
        public void run() {
            corriendo = true;
            while (corriendo) {
                actualizaFisica();
                synchronized (this) {
                    while (pausa) {
                        try {
                            wait();
                        } catch (Exception e) {
                        }
                    }
                }
            }
        }
    }
}
